# {{ connector_name }}

<HideInUI>

This page contains the setup guide and reference information for the [{{ connector_name }}]({docs_link}) source connector.

</HideInUI>

## Prerequisites

## Setup guide

## Set up {{ connector_name }}

Describes actions that should be done on source side before set up in Airbyte

### For Airbyte Cloud:

Describes specific for airbyte cloud set up steps of the connector

### For Airbyte Open Source:

Describes specific for airbyte oss set up steps of the connector

### CONNECTOR_SPECIFIC_FEATURES

Heading fo set up steps which are specific for connector and will be ignored while check.

## Set up the {{ connector_name }} connector in Airbyte

Describes how to set up the connector in Airbyte.

### For Airbyte Cloud:

1. [Log into your Airbyte Cloud](https://cloud.airbyte.com/workspaces) account.
2. Click Sources and then click + New source.
3. On the Set up the source page, select {{ connector_name }} from the Source type dropdown.
4. Enter a name for the {{ connector_name }} connector.
### For Airbyte Open Source:

1. Navigate to the Airbyte Open Source dashboard.
## CONNECTOR_SPECIFIC_FEATURES

Heading for set up steps in Airbyte which are specific for connector and will be ignored while check.

## Supported sync modes

The {{ connector_name }} source connector supports the following [sync modes](https://docs.airbyte.com/cloud/core-concepts/#connection-sync-modes):

## Supported Streams

## CONNECTOR_SPECIFIC_FEATURES

Other related to the connector features, e.g. Include deleted, Supported objects etc.

## IP allow list

If you use Airbyte Cloud and your organization restricts access to specific IPs, add the [Airbyte Cloud IP addresses](https://docs.airbyte.com/platform/operating-airbyte/ip-allowlist) to your allow list.

### Performance considerations

## Data type map

## Limitations & Troubleshooting

### CONNECTOR_SPECIFIC_FEATURES

Specific Limitations & Troubleshooting related to the connector.

### Tutorials

Now that you have set up the {{ connector_name }} source connector, check out the following {{ connector_name }} tutorials:

## Changelog
<details>
  <summary>Expand to review</summary>
</details>
